<?php

/**
 * Plugin Name: Artículo siguiente y previo
 * Description: Añade flechas en los artículos para navegar hacia el artículo siguiente o el anterior.
 * Version: 1.0
 * Author: The Back Room
 */

class nextPrevArticle
{

    private $nextId;
    private $prevId;
    private $paramsBack;
    private $paramsArrows;
    private $filters_base = 'filtros-productos';

    public function __construct()
    {

        $this->fillState();
 
        // STYLES & SCRIPTS
        add_action('wp_enqueue_scripts', array($this, 'stylesScripts'));

        // Shortcodes
        add_shortcode('prev-next-arrows', array($this, 'prev_next_shortcode'));
        add_shortcode('prev-next-products', array($this, 'prev_next_products'));

    }

    // It gets the parameters from the url in order to keep the context of navigation
    public function fillState()
    {

        if ($_SERVER['QUERY_STRING']) {

            $this->paramsBack = '?' . $_SERVER['QUERY_STRING'];
            $this->paramsArrows = '?' . $_SERVER['QUERY_STRING'];

        }

    }

    // Registering Styles & Scripts
    public function stylesScripts()
    {

        global $post;
        $post_t = get_post_type();

        if( $post_t == 'post' || $post_t == 'diy' || $post_t == 'receta' || $post_t == 'product' ) {

            // STYLES
            wp_enqueue_style('next-prev-article', plugin_dir_url(__FILE__) . "style.css", array(), time());

            // SCRIPTS
            wp_enqueue_script('next-prev-article', plugin_dir_url(__FILE__) . 'script.js', array('jquery'), '', true);

        }

    }

    // It returns the next and previous post id based on the taxonomy
    // If the taxonomy is equal to none, it returns posts from all the categories
    public function prevNextPostID( $postid )
    {

        $postlist_args = array(
            'posts_per_page' => -1,
            'orderby' => 'date',
            'order' => 'ASC',
            'post_type' => get_post_type(),
            'tax_query' => array(
                'relation' => 'AND'
            ),
        );

        // Getting filters from the URL
        $queryURL = parse_url(html_entity_decode(esc_url(add_query_arg( $arr_params ))));
        parse_str($queryURL['query'], $getVar);
        $cat = $getVar['categ'];

        if ($cat != '') {

            if (get_post_type() == 'post') {
                $postlist_args['tax_query'][] = array(
                    'taxonomy' => 'categorias_emocionales',
                    'field' => 'term_id',
                    'terms' => array($cat),
                    'operator' => 'IN'
                );
            }
            else {
                $postlist_args['tax_query'][] = array(
                    'taxonomy' => 'tipo_diy',
                    'field' => 'term_id',
                    'terms' => array($cat),
                    'operator' => 'IN'
                );
            }

        }
        if ($getVar['rplato'] != '') {
            $postlist_args['tax_query'][] = array(
                'taxonomy' => 'plato',
                'field' => 'term_id',
                'terms' => array($getVar['rplato']),
                'operator' => 'IN'
            );
        }
        if ($getVar['rdificultad'] != '') {
            $postlist_args['tax_query'][] = array(
                'taxonomy' => 'dificultad',
                'field' => 'term_id',
                'terms' => array($getVar['rdificultad']),
                'operator' => 'IN'
            );
        }
        if ($getVar['rraciones'] != '') {
            $postlist_args['tax_query'][] = array(
                'taxonomy' => 'raciones',
                'field' => 'term_id',
                'terms' => array($getVar['rraciones']),
                'operator' => 'IN'
            );
        }
        if ($getVar['rtiempo'] != '') {
            $postlist_args['tax_query'][] = array(
                'taxonomy' => 'tiempo',
                'field' => 'term_id',
                'terms' => array($getVar['rtiempo']),
                'operator' => 'IN'
            );
        }
        if ($getVar['respeciales'] != '') {
            $postlist_args['tax_query'][] = array(
                'taxonomy' => 'especiales',
                'field' => 'term_id',
                'terms' => array($getVar['respeciales']),
                'operator' => 'IN'
            );
        }
        if ($getVar['page'] != '') {
            $this->urlParams['page'] = $getVar['page'];
        }

        $postlist = get_posts($postlist_args);

        // get ids of posts retrieved from get_posts
        $ids = array();
        foreach ($postlist as $thepost) {
            $ids[] = $thepost->ID;
        }

        // get and echo previous and next post in the same taxonomy
        $thisindex = array_search($postid, $ids);
        $postslist['prev'] = $ids[$thisindex - 1];
        $postslist['next'] = $ids[$thisindex + 1];

        $this->nextId = $ids[$thisindex - 1];
        $this->prevId = $ids[$thisindex + 1];

        return $postslist;

    }

    /**
     * It retrieves the ID of the previous and the next products using the active filters
     * 
     * @param int $productId
     * @param array $filters - Default empty
     * 
     * @return array - ['nextProductId' => , 'prevProductId' => ]
     */
    public function prevNextProductID( $productId, $filters = [] ) {

        $nextPrevId = [
            'nextProductId' => null,
            'prevProductId' => null
        ];


        $productCategoriesRepo = new WCAldi\Repositories\ProductCategoriesRepo;

        $productRepo = new WCAldi\Repositories\ProductRepo();
        $category = $productRepo->getProductCategories( $productId, true )[0];

        if( isset( $filters['order'] ) ) {
            $order = $filters['order'];
        } else {
            $order = 'alphabetical';
        }

        if( isset( $filters['families'] ) ) {
            $families = $filters['families'];
        } else {
            $families = [];
        }

        if( !empty( $category ) ) {
            $products = $productCategoriesRepo->getProductsInCategoriesBBDD( $category['id'], null, 0, $order, $productCategoriesRepo->getCategoriesIdBySlug( $families ) );

            // Get ids of posts retrieved from get_posts
            $ids = array();
            foreach( $products as $p ) {
                $ids[] = $p['id'];
            }

            // get and echo previous and next post in the same taxonomy
            $thisindex = array_search( $productId, $ids );
            $nextPrevId['nextProductId'] = intval( $ids[$thisindex - 1] );
            $nextPrevId['prevProductId'] = intval( $ids[$thisindex + 1] );

        }

        return $nextPrevId;

    }

    public function arrowsCode()
    {

        global $post;

        $this->prevNextPostID( $post->ID );

        $content = '';

        if ( is_single() ) {

            $nexttext = "Siguiente " . get_post_type_object(get_post_type())->labels->singular_name;
            $prevtext = get_post_type_object(get_post_type())->labels->singular_name . " anterior";

            $nextpost = $this->nextId;
            $prevpost = $this->prevId;

            // Checking if we have a next post
            if ($nextpost != '') {

                $thumbnail = get_the_post_thumbnail_url($nextpost, 'medium');

                if ($thumbnail) {
                    $alt_text = get_post_meta($thumbnail->ID, '_wp_attachment_image_alt', true);

                    if (!empty($thumbnail)) {
                        if (!empty($alt_text)) {
                            $alt_text = $alt_text;
                        }
                        else {
                            $alt_text = __('', 'textdomain');
                        }
                    }
                }

                $content .= '<div class="fixed-next-arrow">';
                $content .= '<div class="fixed-arrows-block">';
                $content .= '<a href="' . get_permalink($nextpost) . $this->paramsArrows . '"><i class="fa fa-angle-right" aria-hidden="true"></i>';
                $content .= '<div class="hidden-content-arrows">';
                $content .= '<div class="arrow-image-block"><img src="' . $thumbnail . '" alt="' . $alt_text . '"></div>';
                $content .= '<div class="arrow-title"><small class="post-to-go">' . $nexttext . '</small><div class="arrow-title-text">' . get_the_title($nextpost) . '</div></div>';
                $content .= '</div>';
                $content .= '</a>';
                $content .= '</div>';
                $content .= '</div>';

            }

            // Checking if we have a previous post
            if ($prevpost != '') {

                // Previous
                $thumbnail = get_the_post_thumbnail_url($prevpost, 'medium');

                if ($thumbnail) {
                    $alt_text = get_post_meta($thumbnail->ID, '_wp_attachment_image_alt', true);

                    if (!empty($thumbnail)) {
                        if (!empty($alt_text)) {
                            $alt_text = $alt_text;
                        }
                        else {
                            $alt_text = __('', 'textdomain');
                        }
                        echo '';
                    }
                }
                $content .= '<div class="fixed-prev-arrow">';
                $content .= '<div class="fixed-arrows-block">';
                $content .= '<a href="' . get_permalink($prevpost) . $this->paramsArrows . '"><i class="fa fa-angle-left" aria-hidden="true"></i>';
                $content .= '<div class="hidden-content-arrows left">';
                $content .= '<div class="arrow-image-block left"><img src="' . $thumbnail . '" alt="' . $alt_text . '"></div>';

                $content .= '<div class="arrow-title left"><small class="post-to-go">' . $prevtext . '</small><div class="arrow-title-text">' . get_the_title($prevpost) . '</div></div>';
                $content .= '</div>';
                $content .= '</a>';
                $content .= '</div>';
                $content .= '</div>';

            }

        }
        return $content;

    }

    /**
     * Code for the floating arrows inside the products
     */
    public function arrowsCodeProducts()
    {

        global $post;

        $prettyFilters = new WCAldi\PrettyUrl\PrettyFilters();
        $filters = $prettyFilters->getActiveFilters();
        
        $prevNextProducts = $this->prevNextProductID( $post->ID, $filters );

        $content = '';

        if ( is_single() ) {

            $singularName = get_post_type_object( get_post_type() )->labels->singular_name;

            $nexttext = "Siguiente " . $singularName;
            $prevtext = $singularName . " anterior";

            $nextpost = $prevNextProducts['nextProductId'];
            $prevpost = $prevNextProducts['prevProductId'];

            if( $filters['original_string'] != null && $filters['original_string'] != '' ) {
                $filters_base = $this->filters_base . "/";
            } else {
                $filters_base = '';
            }

            // Checking if we have a next post
            if ($nextpost != '') {

                $thumbnail = get_the_post_thumbnail_url( $nextpost, 'medium' );

                if ($thumbnail) {
                    $alt_text = get_post_meta($thumbnail->ID, '_wp_attachment_image_alt', true);

                    if (!empty($thumbnail)) {
                        if (!empty($alt_text)) {
                            $alt_text = $alt_text;
                        }
                        else {
                            $alt_text = __('', 'textdomain');
                        }
                    }
                }

                $content .= '<div class="fixed-next-arrow">';
                $content .= '<div class="fixed-arrows-block">';
                $content .= '<a href="' . get_permalink($nextpost) . $filters_base . $filters['original_string'] . '"><i class="fa fa-angle-right" aria-hidden="true"></i>';
                $content .= '<div class="hidden-content-arrows">';
                $content .= '<div class="arrow-image-block"><img src="' . $thumbnail . '" alt="' . $alt_text . '"></div>';
                $content .= '<div class="arrow-title"><small class="post-to-go">' . $nexttext . '</small><div class="arrow-title-text">' . get_the_title($nextpost) . '</div></div>';
                $content .= '</div>';
                $content .= '</a>';
                $content .= '</div>';
                $content .= '</div>';

            }

            // Checking if we have a previous post
            if ($prevpost != '') {

                // Previous
                $thumbnail = get_the_post_thumbnail_url($prevpost, 'medium');

                if ($thumbnail) {
                    $alt_text = get_post_meta($thumbnail->ID, '_wp_attachment_image_alt', true);

                    if (!empty($thumbnail)) {
                        if (!empty($alt_text)) {
                            $alt_text = $alt_text;
                        }
                        else {
                            $alt_text = __('', 'textdomain');
                        }
                        echo '';
                    }
                }
                $content .= '<div class="fixed-prev-arrow">';
                $content .= '<div class="fixed-arrows-block">';
                $content .= '<a href="' . get_permalink($prevpost) . $filters_base . $filters['original_string'] . '"><i class="fa fa-angle-left" aria-hidden="true"></i>';
                $content .= '<div class="hidden-content-arrows left">';
                $content .= '<div class="arrow-image-block left"><img src="' . $thumbnail . '" alt="' . $alt_text . '"></div>';

                $content .= '<div class="arrow-title left"><small class="post-to-go">' . $prevtext . '</small><div class="arrow-title-text">' . get_the_title($prevpost) . '</div></div>';
                $content .= '</div>';
                $content .= '</a>';
                $content .= '</div>';
                $content .= '</div>';

            }

        }

        $content .= $this->mobileProductArrows( $prevNextProducts, $filters );

        return $content;

    }

    // An arrow for going back to the archive page
    public function arrowMore()
    {

        $content = '';

        if (is_single()) {

            $content .= '<div class="archive-arrow">';
            $content .= '<div class="archive-arrow-wrapper">';
            $content .= '<a href="' . get_post_type_archive_link(get_post_type()) . $this->paramsBack . '"><div class="archive-arrow-icon"><img src="' . plugin_dir_url(__FILE__) . 'icon-return.png" alt="Icono de flecha apuntando a la izquierda"></div></a>';
            $content .= '<div class="archive-arrow-text"><a href="' . get_post_type_archive_link(get_post_type()) . $this->paramsBack . '">Volver a ' . get_post_type_object(get_post_type())->labels->name . '</a></div>';
            $content .= '</div>';
            $content .= '</div>';

        }

        return $content;

    }

    // Mobile arrows
    public function mobileArrows()
    {

        $content = '';

        $content .= '<div class="archive-arrow-mobile">';
        if ($this->prevId != '') {
            $content .= '<div class="left"><a href="' . get_permalink($this->prevId) . $this->paramsArrows . '"><i class="fa fa-angle-left" aria-hidden="true"></i></a></div>';
        }
        $content .= '<div class="button"><a href="' . get_post_type_archive_link(get_post_type()) . $this->paramsBack . '">MÁS ' . get_post_type_object(get_post_type())->labels->name . '</a></div>';
        if ($this->nextId != '') {
            $content .= '<div class="right"><a href="' . get_permalink($this->nextId) . $this->paramsArrows . '"><i class="fa fa-angle-right" aria-hidden="true"></i></a></div>';
        }
        $content .= '</div>';

        return $content;

    }

    // Mobile product arrows
    public function mobileProductArrows( $prevNextProducts, $filters )
    {

        $content = '';

        if( $filters['original_string'] != null && $filters['original_string'] != '' ) {
            $filters_base = $this->filters_base . "/";
        } else {
            $filters_base = '';
        }

        $content .= '<div class="archive-arrow-mobile">';
        if ( !empty($prevNextProducts['prevProductId']) ) {
            $content .= '<div class="left"><a href="' . get_permalink($prevNextProducts['prevProductId']) . $filters_base .  $this->paramsArrows . '"><i class="fa fa-angle-left" aria-hidden="true"></i></a></div>';
        }
        if ( !empty($prevNextProducts['nextProductId']) ) {
            $content .= '<div class="right"><a href="' . get_permalink($prevNextProducts['nextProductId']) . $filters_base . $this->paramsArrows . '"><i class="fa fa-angle-right" aria-hidden="true"></i></a></div>';
        }
        $content .= '</div>';

        return $content;

    }

    // It prepares the code that the shortcode will show
    public function prev_next_shortcode()
    {

        echo $this->arrowsCode();
        echo $this->arrowMore();
        echo $this->mobileArrows();

    }

    // It prepares the code that the shortcode for the products will show
    public function prev_next_products()
    {

        echo $this->arrowsCodeProducts();

    }

}

$npa = new nextPrevArticle();